package nl.amis.jfall2009.jsf.beans;

import java.io.Serializable;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.ejb.Local;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.SelectItem;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import nl.amis.jfall2009.ejb.entity.Sac;
import nl.amis.jfall2009.ejb.session.JFall2009SessionBean;
import nl.amis.jfall2009.enums.ConstellationEnum;
import nl.amis.jfall2009.enums.DeepSkyTypeEnum;

@EJB(name="jf2009Session", beanInterface=JFall2009SessionBean.class)
@ManagedBean(name="jf2009mb")
@Local
@SessionScoped
public class JFall2009ManagedBean implements Serializable {
    private static final long serialVersionUID = 1L;

    private JFall2009SessionBean jf2009Session;
    private DataModel sacModel = new ListDataModel();
    private String name = "";
    private Integer ra1;
    private Integer ra2;
    private Integer dec1;
    private Integer dec2;
    private Double mag;
    private String constellation = "";
    private String sacType = "";
    private List<SelectItem> constellations, sacTypes, catalogs, hours, degrees, magnitudes;
    private String degPattern = "00";
    private String minPattern = "00";
    private String secPattern = "00.#";
    private DecimalFormat degFormat = new DecimalFormat(degPattern);
    private DecimalFormat minFormat = new DecimalFormat(minPattern);
    private DecimalFormat secFormat = new DecimalFormat(secPattern);
    private Sac newSac;

    @PostConstruct
    public void init() {
        try {
            InitialContext ic = new InitialContext();
            jf2009Session = (JFall2009SessionBean) ic.lookup("java:comp/env/jf2009Session");
            reload();
        } catch (NamingException ex) {
            Logger.getLogger(JFall2009ManagedBean.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void reload() {
        List<Sac> sacList = jf2009Session.getSacObjects(name, ra1, ra2, dec1, dec2, mag, constellation, sacType);
        sacModel.setWrappedData(sacList);
    }

    public String filter() {
        reload();
        return "index";
    }

    public String size(Double size) {
        double seconds = size % 60;
        int minutes = (int) ((size - seconds) / 60) % 60;
        int degs = (int) (size - (seconds + minutes * 60)) / 3600;
        return degFormat.format(degs) + "⁰ " + minFormat.format(minutes) + "' " + secFormat.format(seconds) + "\"";
    }

    public String formattedNumber(Number number) {
        if (number instanceof Integer) {
            return (number.intValue() < 10 ? "0" + number.intValue() : "" + number.intValue());
        } else {
            return (number.doubleValue() < 10 ? "0" + number.doubleValue() : "" + number.doubleValue());
        }
    }

    /**
     * Retrieves the list of SAC Objects
     *
     * @return a list of SAC Objects
     */
    public DataModel getSacObjects() {
        return sacModel;
    }

    /**
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the ra1
     */
    public Integer getRa1() {
        return ra1;
    }

    /**
     * @param ra1 the ra1 to set
     */
    public void setRa1(Integer ra1) {
        this.ra1 = ra1;
    }

    /**
     * @return the ra2
     */
    public Integer getRa2() {
        return ra2;
    }

    /**
     * @param ra2 the ra2 to set
     */
    public void setRa2(Integer ra2) {
        this.ra2 = ra2;
    }

    /**
     * @return the dec1
     */
    public Integer getDec1() {
        return dec1;
    }

    /**
     * @param dec1 the dec1 to set
     */
    public void setDec1(Integer dec1) {
        this.dec1 = dec1;
    }

    /**
     * @return the dec2
     */
    public Integer getDec2() {
        return dec2;
    }

    /**
     * @param dec2 the dec2 to set
     */
    public void setDec2(Integer dec2) {
        this.dec2 = dec2;
    }

    /**
     * @return the mag
     */
    public Double getMag() {
        return mag;
    }

    /**
     * @param mag the mag to set
     */
    public void setMag(Double mag) {
        this.mag = mag;
    }

    /**
     * @return the constellation
     */
    public String getConstellation() {
        return constellation;
    }

    /**
     * @param constellation the constellation to set
     */
    public void setConstellation(String constellation) {
        this.constellation = constellation;
    }

    /**
     * @return the sacType
     */
    public String getSacType() {
        return sacType;
    }

    /**
     * @param sacType the sacType to set
     */
    public void setSacType(String sacType) {
        this.sacType = sacType;
    }

    /**
     * @return the constellations
     */
    public List<SelectItem> getConstellations() {
        if (constellations == null) {
            constellations = new ArrayList<SelectItem>();
            constellations.add(new SelectItem(""));
            for (ConstellationEnum constellationEnum : ConstellationEnum.values()) {
                constellations.add(new SelectItem(constellationEnum.getName()));
            }
        }
        return constellations;
    }

    /**
     * @return the sacTypes
     */
    public List<SelectItem> getSacTypes() {
        if (sacTypes == null) {
            sacTypes = new ArrayList<SelectItem>();
            sacTypes.add(new SelectItem(""));
            for (DeepSkyTypeEnum deepSkyTypeEnum : DeepSkyTypeEnum.values()) {
                sacTypes.add(new SelectItem(deepSkyTypeEnum.getName()));
            }
        }
        return sacTypes;
    }

    /**
     * @return the catalogs
     */
    public List<SelectItem> getCatalogs() {
        if (catalogs == null) {
            catalogs = new ArrayList<SelectItem>();
            catalogs.add(new SelectItem(""));
            for (String catalog : jf2009Session.getCatalogs()) {
                catalogs.add(new SelectItem(catalog));
            }
        }
        return catalogs;
    }

    /**
     * @return the hours
     */
    public List<SelectItem> getHours() {
        if (hours == null) {
            hours = new ArrayList<SelectItem>();
            hours.add(new SelectItem(""));
            for (int i = 0; i < 25; i++) {
                hours.add(new SelectItem(String.valueOf(i)));
            }
        }
        return hours;
    }

    /**
     * @return the degrees
     */
    public List<SelectItem> getDegrees() {
        if (degrees == null) {
            degrees = new ArrayList<SelectItem>();
            degrees.add(new SelectItem(""));
            for (int i = 90; i > -91; i--) {
                degrees.add(new SelectItem(String.valueOf(i)));
            }
        }
        return degrees;
    }

    /**
     * @return the magnitudes
     */
    public List<SelectItem> getMagnitudes() {
        if (magnitudes == null) {
            magnitudes = new ArrayList<SelectItem>();
            magnitudes.add(new SelectItem(""));
            for (int i = 0; i < 20; i++) {
                magnitudes.add(new SelectItem(String.valueOf(i + 0.0)));
                magnitudes.add(new SelectItem(String.valueOf(i + 0.5)));
            }
        }
        return magnitudes;
    }

    public String initializeNewSac() {
        setNewSac(new Sac());
        return "add";
    }

    public String saveAction() {
        jf2009Session.addSac(newSac);
        return "index";
    }

    public String cancelAction() {
        return "index";
    }

    /**
     * @return the newSac
     */
    public Sac getNewSac() {
        return newSac;
    }

    /**
     * @param newSac the newSac to set
     */
    public void setNewSac(Sac newSac) {
        this.newSac = newSac;
    }
}